package logic

import (
	"context"
	"fmt"

	"git.i2edu.net/i2/i2-bill-erp/internal/svc"
	"git.i2edu.net/i2/i2-bill-erp/model"
	"git.i2edu.net/i2/i2-bill-erp/transform"

	"git.i2edu.net/i2/go-zero/core/logx"
)

type GetErpMktNetWorkDetailLogic struct {
	ctx    context.Context
	svcCtx *svc.ServiceContext
	logx.Logger
}

func NewGetErpMktNetWorkDetailLogic(ctx context.Context, svcCtx *svc.ServiceContext) *GetErpMktNetWorkDetailLogic {
	return &GetErpMktNetWorkDetailLogic{
		ctx:    ctx,
		svcCtx: svcCtx,
		Logger: logx.WithContext(ctx),
	}
}

// 获取erp 渠道细分树
func (l *GetErpMktNetWorkDetailLogic) GetErpMktNetWorkDetail(in *transform.ErpMktNetWorkDetailReq) (*transform.ErpMktNetWorkDetailRes, error) {
	var items []model.MktNetworkDetail
	err := l.svcCtx.SqlConn.QueryRowsPartial(&items, fmt.Sprintf("select id, name, parent, inheritance from mkt_network_detail where id in (%v) and del_flag!=1", in.Ids))
	if err != nil {
		return nil, err
	}
	tc := transform.ErpMktNetWorkDetailRes{}
	for i := range items {
		tc.Items = append(tc.Items, &transform.ErpMktNetWorkDetail{
			Id:          items[i].Id,
			Text:        items[i].Name.String,
			Parent:      items[i].Parent.Int64,
			Inheritance: items[i].Inheritance.String,
		})
	}
	return &tc, nil
}
