package model

import (
	"database/sql"
	"fmt"
	"strings"

	"git.i2edu.net/i2/go-zero/core/stores/sqlc"
	"git.i2edu.net/i2/go-zero/core/stores/sqlx"
	"git.i2edu.net/i2/go-zero/core/stringx"
	"git.i2edu.net/i2/go-zero/tools/goctl/model/sql/builderx"
)

var (
	mktNetworkDetailFieldNames          = builderx.RawFieldNames(&MktNetworkDetail{})
	mktNetworkDetailRows                = strings.Join(mktNetworkDetailFieldNames, ",")
	mktNetworkDetailRowsExpectAutoSet   = strings.Join(stringx.Remove(mktNetworkDetailFieldNames, "`id`", "`create_time`", "`update_time`"), ",")
	mktNetworkDetailRowsWithPlaceHolder = strings.Join(stringx.Remove(mktNetworkDetailFieldNames, "`id`", "`create_time`", "`update_time`"), "=?,") + "=?"
)

type (
	MktNetworkDetailModel interface {
		Insert(data MktNetworkDetail) (sql.Result, error)
		FindOne(id int64) (*MktNetworkDetail, error)
		Update(data MktNetworkDetail) error
		Delete(id int64) error
	}

	defaultMktNetworkDetailModel struct {
		conn  sqlx.SqlConn
		table string
	}

	MktNetworkDetail struct {
		LastUpdateTime sql.NullTime    `db:"last_update_time" json:"last_update_time"`
		CreateBy       sql.NullString  `db:"create_by" json:"create_by"`
		LastUpdateBy   sql.NullString  `db:"last_update_by" json:"last_update_by"`
		Parent         sql.NullInt64   `db:"parent" json:"parent"`
		Inheritance    sql.NullString  `db:"inheritance" json:"inheritance"`
		Remake         sql.NullString  `db:"remake" json:"remake"`
		CreateTime     sql.NullTime    `db:"create_time" json:"create_time"`
		DelFlag        sql.NullFloat64 `db:"del_flag" json:"del_flag"`
		Qudao          sql.NullInt64   `db:"qudao" json:"qudao"`
		Id             int64           `db:"id" json:"id"`
		Name           sql.NullString  `db:"name" json:"name"`
		ChannelType    sql.NullInt64   `db:"channel_type" json:"channel_type"`
		QueryType      sql.NullInt64   `db:"query_type" json:"query_type"`
	}
)

func NewMktNetworkDetailModel(conn sqlx.SqlConn) MktNetworkDetailModel {
	return &defaultMktNetworkDetailModel{
		conn:  conn,
		table: "`mkt_network_detail`",
	}
}

func (m *defaultMktNetworkDetailModel) Insert(data MktNetworkDetail) (sql.Result, error) {
	query := fmt.Sprintf("insert into %s (%s) values (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)", m.table, mktNetworkDetailRowsExpectAutoSet)
	ret, err := m.conn.Exec(query, data.LastUpdateTime, data.CreateBy, data.LastUpdateBy, data.Parent, data.Inheritance, data.Remake, data.DelFlag, data.Qudao, data.Name, data.ChannelType, data.QueryType)
	return ret, err
}

func (m *defaultMktNetworkDetailModel) FindOne(id int64) (*MktNetworkDetail, error) {
	query := fmt.Sprintf("select %s from %s where `id` = ? limit 1", mktNetworkDetailRows, m.table)
	var resp MktNetworkDetail
	err := m.conn.QueryRow(&resp, query, id)
	switch err {
	case nil:
		return &resp, nil
	case sqlc.ErrNotFound:
		return nil, ErrNotFound
	default:
		return nil, err
	}
}

func (m *defaultMktNetworkDetailModel) Update(data MktNetworkDetail) error {
	query := fmt.Sprintf("update %s set %s where `id` = ?", m.table, mktNetworkDetailRowsWithPlaceHolder)
	_, err := m.conn.Exec(query, data.LastUpdateTime, data.CreateBy, data.LastUpdateBy, data.Parent, data.Inheritance, data.Remake, data.DelFlag, data.Qudao, data.Name, data.ChannelType, data.QueryType, data.Id)
	return err
}

func (m *defaultMktNetworkDetailModel) Delete(id int64) error {
	query := fmt.Sprintf("delete from %s where `id` = ?", m.table)
	_, err := m.conn.Exec(query, id)
	return err
}
